==> lhcsmapi-1.4.53.tar.gz: lhcsmapi-1.4.53/lhcsmapi/gui/pc/R600AFgcPmSearchBaseModule.py <==
from typing import List, Tuple

import numpy as np

from lhcsmapi.Time import Time
from lhcsmapi.pyedsl.QueryBuilder import QueryBuilder
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.gui.pc.FgcPmSearchBaseModule import FgcPmSearchBaseModule


def combine_coupled_fgc_pm_entries(fgc_pm_entries: List[Tuple[str, int]], circuit_prefixes: List[str]
                                   ) -> List[Tuple[str, int]]:
    """ Function combining coupled FGC pm entries into a single list of tuples

    :param fgc_pm_entries: a list of FGC PM source and timestamp tuples
    :param circuit_prefixes: a list of circuit prefixes
    :return: a list of source and timestamp tuples
    """
    out_pm_entries = []
    for fgc_pm_entry in fgc_pm_entries:
        circuit_name = fgc_pm_entry[0]
        timestamp = fgc_pm_entry[1]
        # find coupled circuit name
        if circuit_prefixes[0] in circuit_name:
            coupled_circuit_name = circuit_name.replace(circuit_prefixes[0], circuit_prefixes[1])
        else:
            coupled_circuit_name = circuit_name.replace(circuit_prefixes[1], circuit_prefixes[0])

        # find coupled circuit name
        indices_coupled = [i for i, tupl in enumerate(fgc_pm_entries) if tupl[0] == coupled_circuit_name]

        # check if timestamp is within 2 s
        found_index = -1
        for index_coupled in indices_coupled:
            if abs(timestamp - fgc_pm_entries[index_coupled][1]) <= 2e9:
                found_index = index_coupled

        if found_index == -1:
            out_pm_entries.append((circuit_name, timestamp))
        else:
            out_pm_entries.append((circuit_name, timestamp, coupled_circuit_name, fgc_pm_entries[found_index][1]))
            fgc_pm_entries.pop(found_index)

    return out_pm_entries


def query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names: List[str]) -> List[Tuple[str, int]]:
    """ Method querying FGC PM source and timestamp tuple and taking only those corresponding to the given circuits

    :param start_time: start time for a query
    :param end_time: end time for a query
    :param circuit_names: list of circuit names for which
    :return: a list of PM source and timestamp tuples corresponding to circuit names
    """
    # Query PM for the specified period of time

    metadata = SignalMetadata.get_circuit_signal_database_metadata('IPQ', circuit_names[0], 'PC', 'PM',
                                                                   timestamp_query=start_time)
    source_timestamp_df = QueryBuilder().with_pm() \
        .with_duration(t_start=start_time, t_end=end_time) \
        .with_query_parameters(system='FGC', className=metadata['className'], source='*') \
        .event_query().df

    # Keep only sources corresponding to circuit names
    pc_names = [SignalMetadata.get_circuit_signal_database_metadata('600A', circuit_name, 'PC', 'PM')['source']
                for circuit_name in circuit_names]
    mask = source_timestamp_df['source'].isin(pc_names)
    source_timestamp_mask_df = source_timestamp_df[mask]

    # Convert PC names to circuit names
    output = []
    for index, row in source_timestamp_mask_df.iterrows():
        pc_name = row['source']
        pc_name_split = pc_name.split('.')
        circuit_name = '%s.%s' % (pc_name_split[-2], pc_name_split[-1])
        output.append((circuit_name, row['timestamp']))

    return output


class R600AFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for 600A FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names_all = SignalMetadata.get_circuit_names('600A')
        circuit_names = filter(lambda x: circuit_name in x, circuit_names_all)

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, list(circuit_names))

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output


class R600ARcdoFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for RCD-RCO FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def get_circuit_names(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> List[str]:
        """ Method returning circuit name for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected circuit name
        """
        if len(fgc_pm_entries[index]) == 2:
            circuit_name = fgc_pm_entries[index][0]
            return [circuit_name if 'RCD' in circuit_name else circuit_name.replace('RCO', 'RCD'),
                    circuit_name if 'RCO' in circuit_name else circuit_name.replace('RCD', 'RCO')]
        else:
            return sorted(list([fgc_pm_entries[index][0], fgc_pm_entries[index][2]]))

    def get_circuit_timestamps(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> Tuple[float, float]:
        """ Method returning a tuple of timestamps for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected timestamp
        """
        if len(fgc_pm_entries[index]) == 2:
            if 'RCD' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], np.nan
            else:
                return np.nan, fgc_pm_entries[index][1]
        else:
            if 'RCD' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], fgc_pm_entries[index][3]
            else:
                return fgc_pm_entries[index][3], fgc_pm_entries[index][1]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names = [circuit_name.replace('RC', 'RCD'), circuit_name.replace('RC', 'RCO')]

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names)
        output_coupled = combine_coupled_fgc_pm_entries(output, circuit_prefixes=['RCD', 'RCO'])

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output_coupled


class R600ARcbxhvFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for RCBX FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def get_circuit_names(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> List[str]:
        """ Method returning circuit name for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected list of circuit names
        """
        if len(fgc_pm_entries[index]) == 2:
            circuit_name = fgc_pm_entries[index][0]
            return [circuit_name if 'RCBXH' in circuit_name else circuit_name.replace('RCBXV', 'RCBXH'),
                    circuit_name if 'RCBXV' in circuit_name else circuit_name.replace('RCBXH', 'RCBXV')]
        else:
            return sorted(list([fgc_pm_entries[index][0], fgc_pm_entries[index][2]]))

    def get_circuit_timestamps(self, index, fgc_pm_entries: List[Tuple[str, int]]) -> Tuple[float, float]:
        """ Method returning a tuple of timestamps for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected timestamp
        """
        if len(fgc_pm_entries[index]) == 2:
            if 'RCBXH' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], np.nan
            else:
                return np.nan, fgc_pm_entries[index][1]
        else:
            if 'RCBXH' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], fgc_pm_entries[index][3]
            else:
                return fgc_pm_entries[index][3], fgc_pm_entries[index][1]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names = [circuit_name.replace('RCBX', 'RCBXH'),
                         circuit_name.replace('RCBX', 'RCBXV')]

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.value = 0
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names)
        output_coupled = combine_coupled_fgc_pm_entries(output, circuit_prefixes=['RCBXH', 'RCBXV'])

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output_coupled

==> lhcsmapi-1.4.58.tar.gz: lhcsmapi-1.4.58/lhcsmapi/gui/pc/R600AFgcPmSearchBaseModule.py <==
from typing import List, Tuple

import numpy as np

from lhcsmapi.Time import Time
from lhcsmapi.pyedsl.QueryBuilder import QueryBuilder
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.gui.pc.FgcPmSearchBaseModule import FgcPmSearchBaseModule


def combine_coupled_fgc_pm_entries(fgc_pm_entries: List[Tuple[str, int]], circuit_prefixes: List[str]
                                   ) -> List[Tuple[str, int]]:
    """ Function combining coupled FGC pm entries into a single list of tuples

    :param fgc_pm_entries: a list of FGC PM source and timestamp tuples
    :param circuit_prefixes: a list of circuit prefixes
    :return: a list of source and timestamp tuples
    """
    out_pm_entries = []
    for fgc_pm_entry in fgc_pm_entries:
        circuit_name = fgc_pm_entry[0]
        timestamp = fgc_pm_entry[1]
        # find coupled circuit name
        if circuit_prefixes[0] in circuit_name:
            coupled_circuit_name = circuit_name.replace(circuit_prefixes[0], circuit_prefixes[1])
        else:
            coupled_circuit_name = circuit_name.replace(circuit_prefixes[1], circuit_prefixes[0])

        # find coupled circuit name
        indices_coupled = [i for i, tupl in enumerate(fgc_pm_entries) if tupl[0] == coupled_circuit_name]

        # check if timestamp is within 2 s
        found_index = -1
        for index_coupled in indices_coupled:
            if abs(timestamp - fgc_pm_entries[index_coupled][1]) <= 2e9:
                found_index = index_coupled

        if found_index == -1:
            out_pm_entries.append((circuit_name, timestamp))
        else:
            out_pm_entries.append((circuit_name, timestamp, coupled_circuit_name, fgc_pm_entries[found_index][1]))
            fgc_pm_entries.pop(found_index)

    return out_pm_entries


def query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names: List[str]) -> List[Tuple[str, int]]:
    """ Method querying FGC PM source and timestamp tuple and taking only those corresponding to the given circuits

    :param start_time: start time for a query
    :param end_time: end time for a query
    :param circuit_names: list of circuit names for which
    :return: a list of PM source and timestamp tuples corresponding to circuit names
    """
    # Query PM for the specified period of time

    metadata = SignalMetadata.get_circuit_signal_database_metadata('600A', circuit_names[0], 'PC', 'PM',
                                                                   timestamp_query=start_time)
    source_timestamp_df = QueryBuilder().with_pm() \
        .with_duration(t_start=start_time, t_end=end_time) \
        .with_query_parameters(system='FGC', className=metadata['className'], source='*') \
        .event_query().df

    # Keep only sources corresponding to circuit names
    pc_names = [SignalMetadata.get_circuit_signal_database_metadata('600A', circuit_name, 'PC', 'PM')['source']
                for circuit_name in circuit_names]
    mask = source_timestamp_df['source'].isin(pc_names)
    source_timestamp_mask_df = source_timestamp_df[mask]

    # Convert PC names to circuit names
    output = []
    for index, row in source_timestamp_mask_df.iterrows():
        pc_name = row['source']
        pc_name_split = pc_name.split('.')
        circuit_name = '%s.%s' % (pc_name_split[-2], pc_name_split[-1])
        output.append((circuit_name, row['timestamp']))

    return output


class R600AFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for 600A FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names_all = SignalMetadata.get_circuit_names('600A')
        circuit_names = filter(lambda x: circuit_name in x, circuit_names_all)

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, list(circuit_names))

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output


class R600ARcdoFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for RCD-RCO FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def get_circuit_names(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> List[str]:
        """ Method returning circuit name for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected circuit name
        """
        if len(fgc_pm_entries[index]) == 2:
            circuit_name = fgc_pm_entries[index][0]
            return [circuit_name if 'RCD' in circuit_name else circuit_name.replace('RCO', 'RCD'),
                    circuit_name if 'RCO' in circuit_name else circuit_name.replace('RCD', 'RCO')]
        else:
            return sorted(list([fgc_pm_entries[index][0], fgc_pm_entries[index][2]]))

    def get_circuit_timestamps(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> Tuple[float, float]:
        """ Method returning a tuple of timestamps for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected timestamp
        """
        if len(fgc_pm_entries[index]) == 2:
            if 'RCD' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], np.nan
            else:
                return np.nan, fgc_pm_entries[index][1]
        else:
            if 'RCD' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], fgc_pm_entries[index][3]
            else:
                return fgc_pm_entries[index][3], fgc_pm_entries[index][1]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names = [circuit_name.replace('RC', 'RCD'), circuit_name.replace('RC', 'RCO')]

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names)
        output_coupled = combine_coupled_fgc_pm_entries(output, circuit_prefixes=['RCD', 'RCO'])

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output_coupled


class R600ARcbxhvFgcPmSearchBaseModule(FgcPmSearchBaseModule):
    """ A class with methods to search for RCBX FGC PM events to fill-in a search field in the FGC PM search module.

    """

    def get_circuit_names(self, index: int, fgc_pm_entries: List[Tuple[str, int]]) -> List[str]:
        """ Method returning circuit name for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected list of circuit names
        """
        if len(fgc_pm_entries[index]) == 2:
            circuit_name = fgc_pm_entries[index][0]
            return [circuit_name if 'RCBXH' in circuit_name else circuit_name.replace('RCBXV', 'RCBXH'),
                    circuit_name if 'RCBXV' in circuit_name else circuit_name.replace('RCBXH', 'RCBXV')]
        else:
            return sorted(list([fgc_pm_entries[index][0], fgc_pm_entries[index][2]]))

    def get_circuit_timestamps(self, index, fgc_pm_entries: List[Tuple[str, int]]) -> Tuple[float, float]:
        """ Method returning a tuple of timestamps for a selected element in the list

        :param index: index of a selected element in the FGC PM list
        :param fgc_pm_entries: a list of all source and timestamp FGC PM tuples
        :return: a selected timestamp
        """
        if len(fgc_pm_entries[index]) == 2:
            if 'RCBXH' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], np.nan
            else:
                return np.nan, fgc_pm_entries[index][1]
        else:
            if 'RCBXH' in fgc_pm_entries[index][0]:
                return fgc_pm_entries[index][1], fgc_pm_entries[index][3]
            else:
                return fgc_pm_entries[index][3], fgc_pm_entries[index][1]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name: str) -> List[Tuple[str, int]]:
        """ Method finding FGC PM entries with QPS for a given time range and circuit name

        :param start_time: start time for the query
        :param end_time: end time for the query
        :param circuit_name: circuit name
        :return: a list of FGC source and timestamp tuples
        """
        circuit_names = [circuit_name.replace('RCBX', 'RCBXH'),
                         circuit_name.replace('RCBX', 'RCBXV')]

        self.lbl_search.value = 'Querying PM for FGC events'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = 1
        self.progress_search.value = 0
        self.progress_search.description = ''

        output = query_pm_convert_pc_to_circuit_name(start_time, end_time, circuit_names)
        output_coupled = combine_coupled_fgc_pm_entries(output, circuit_prefixes=['RCBXH', 'RCBXV'])

        self.lbl_search.value = 'Query completed'
        self.progress_search.layout.visibility = 'visible'
        self.progress_search.value = 1
        self.progress_search.description = ''

        return output_coupled
